#!/bin/bash
set +o posix
# $ bash music-rater.sh 1000 irish-samples-1.txt irish-1

N="$1" # currently needs to be a multiple of 10
CORPUS1="$2" # "/home/gwern/wiki/docs/ai/2019-03-06-gpt2-poetry-1000samples.txt"
CORPUS2="$3"
JSON="$4"

encode() {
    TMP_FILE=$(mktemp /tmp/XXXXXX.txt)
    echo "$@" >> $TMP_FILE
    ENCODED=$(PYTHONPATH=src python3 encode2.py --model_name 117M $TMP_FILE)
    echo "$ENCODED"; rm "$TMP_FILE"; }
export -f encode

generateJson() {
    echo "{\"query\": [0,0], \"sample0\": $2, \"sample1\": $3, \"best\": $1}," >> $JSON-encoded.json;
    ## Store a backup copy of the plain text for easier consultation
    echo "{\"query\": [0,0], \"sample0\": $4, \"sample1\": $5, \"best\": $1}," >> $JSON-text.json;
}
generateJsonBroken() {
    echo "{\"query\": [0,0], \"sample0\": $2, \"sample1\": $3, \"best\": $1, \"broken\": 1}," >> $JSON-encoded.json;
    ## Store a backup copy of the plain text for easier consultation
    echo "{\"query\": [0,0], \"sample0\": $4, \"sample1\": $5, \"best\": $1, \"broken\": 1}," >> $JSON-text.json;
}


rm -rf /tmp/music-samples-*/; mkdir /tmp/music-samples-1/ /tmp/music-samples-2/
cat "$CORPUS1" | sed -e 's/===.*/<|endoftext|>/g' -e 's/⏎/\n/g' | \
    csplit --quiet --elide-empty-files --suppress-matched --prefix /tmp/music-samples-1/sample- - '/<|endoftext|>/' '{*}'
cat "$CORPUS2" | sed -e 's/===.*/<|endoftext|>/g' -e 's/⏎/\n/g' | \
    csplit --quiet --elide-empty-files --suppress-matched --prefix /tmp/music-samples-2/sample- - '/<|endoftext|>/' '{*}'

# Pre-compute all versions for speed; this also helps debugging since all stages can be inspected on disk in /tmp/music-samples/
generateEncoded() {
    POEM="$@"
    echo "Starting encoding: $@"
    FIRST=$(cat $POEM)
    encode "<|endoftext|>$FIRST\n<|endoftext|>" >> $POEM.encoded
    abc2midi "$POEM" -o $POEM.midi -Q 130
    timidity -A125 -G5-20 $POEM.midi -Ow -o $POEM.wav
}
export -f generateEncoded
ls /tmp/music-samples-1/sample-* | head -$N | parallel generateEncoded
ls /tmp/music-samples-2/sample-* | head -$N | parallel generateEncoded

filterMusic () {
    fgrep -i -e 'a a a a' -e 'b b b b' -e "a' a' a' a'" -e 'EDEDEDEDEDE' -e 'ED{ED{ED{ED{ED{' -e 'g g g g g' \
          -e 'abbabbaba' -e '=a=a=a=a=a=a=a' -e 'G,G,G,G,G,G,G,G' -e '|=b=a=g=b=a=g|' -e '2|G2G2G2G2|' \
          -e '|G2G2G2G2G2G2|' -e '|G,2G,2G,2|G,' -e '=E=F=G=F=F=F=F=F=E=F' -e ',G,|G,A,B,G,A,B,G,|' \
          -e '|=G=E=E=G=A=B=c=A|' -e '|^g=f^d^c^a^a|^g=f^' -e '^f=f=f^f=f^f=f^d=f^f=f^' -e '2G2E2|C2G2A2G2E2|' \
          -e 'f=e|=f=g=f=e=g=f=e=g=f' -e 'f^d|^c^d^f^g^f^d|^c' -e '=A|=c=A=A2=c=A=G=A|=c=A=A2=c=A=G=A|' \
          -e '=G|=G=G=G=G=G=G|' -e '|E2D2|E2D2|E2D2|' -e '=e=d=g=d=e=d=g=d|=e=d=g' -e '2G,|G,2G,A,2G,A,2G,|C2G' \
          -e '|EDDD|EDDD|EDDD|' -e '|C2ED2E|C2ED2E|' -e '|G2G2A2|G2G2A2|G' -e 'b=a=g|=b=a=g=b=a=g|=b=a' \
          -e '=g|=a=f=a=g=e=g|=a' -e 'B=G=A|=B=c=d=B=c=B=G=A|=B=c=d' -e '|G,EEEDEGG2|G,EEEDEGG2|G' \
          -e '|=G=G=G=G=G=G=G=G|' -e '=G=G=G=G=G=G=G=G=G=G' -e '|EDEEDE|EDEEDE|EDEDED|' -e '|=c=c=c=g=c=e=c=g=c|=c' \
          -e "=e'|=e'=a'=a'=a'=g'=e'|=e'" -e ',2B,2B,2|B,2B,2B,2|' -e '=G,|=G,=A,=A,=A,=G,=G,|=G,' \
          -e '|=c=d=c=B=c=d=e=d|=c=d=c=B=c=d=e=d|' -e 'G2|G2G2G2|G2' -e "|=c=e=g=c'=g=e|=c=e=g=c'=g=e|" \
          -e "|=c'=d'=c'=c'=a=g|=c'=d'=c'=c'=a=g|" -e 'f=a=a|=c=e=e=f=a=a|=c=e' -e '^g|^g=f=f^d^d^c^c^g|^g=f' \
          -e '|=e>=f=g>=e=d>=c=c>=d|=e>=' -e 'g=e=g|=a=e=e=a=a=g=e=g|=a=e=' -e '|^A=c^c^A^G=F|^A=c^c^A^G=F|' \
          -e '|gggggg|gggggg|' -e 'A2|=A2G2A2|=A' -e "2D2|G2D2G2D2|G2" -e 'a=a|=a=a=a=a=a=a|=' -e '=g|=d=f=f=f=f=g|=d=f=f=f=f=g|=' \
          -e 'G,|G,2G,G,|G,2G,G,|G' -e '=G|=G=G=G=G=G=G=G|=G' -e '|=d=f=g=f=g=f=d=c|=d=f=g=f=g=f=d=c|' -e '|=G=G=G=G|=G=G=G=G|' \
          -e '=b=d=b=d=b=d=b=d=b=d' -e '|D2E2D2|E2D2C2|D2E2D2|' -e '|=E=G=G=E=F=A=A=F|=E=G=G=E=F=A=A=F|' -e '|G,A,G,A,G,A,G,A,|G,A,G,A,' \
          -e '|A,A,A,A,A,A,A,|A,A' -e '|ECEECECECECECECECECECECECE' -e '|(G,G,G,G,G' -e 'G,G,G,G,G,G,G' -e '|EDDDED|EDDDED|' \
          -e '|=G,2=G,2=G,2|=G,2' -e 'G=A=c=G=A=c=G=A=c=G=A=c=G=A' -e '|=A=F=A=F=A=F=A=F|=A=F=A' -e ',|G,2A,2G,2G,A,|G,2A,2G,2A' \
          -e '|=G=E=E=G=G=E=G=E|=G=E=' -e '|^G^G^G^G^G^G^G^G^G|' -e '|C2CD2E|C2CD2E|C' -e 'g=g^g|^g^g=g^g=g^g=g^g|' \
          -e '|=c=g=e=g=c=g=e=g|=c=g=e=g=c=g=e=g|' -e '|=G=A=G=c=G=G|=G=A=G=c=A=G|' -e 'g=g|=d=g=g=d=g=g|' -e '|=a=g=g=g=f=e|=d=g=g=e=g=g|' \
          -e 'g=g|=a=g=f=e=g=g|=d' -e "a=g|=f=g=a=c'=a=g|=f" -e '|D2E2D2D2|D2E2D2D2|D2' -e 'D|GDDD=FDDD|GDDD=FDDD|G' -e 'G|=G=A=G=G2=G|=G=A=G=G2=G|' \
          -e '=G2|=G2=G2=G2|=G2' -e 'D2|=D2=D2=C2|=C2=D2=D2|' -e '=D2=D2|=D2=D2=D2|' -e '|=a=f=a=a=f=a|=a=f=a=a=f=a|' -e '|=g=e=g=g=e=g|=g=e=g=g=e=g|' \
          -e '|^c2^A2^A^G^A=c|^c2^A2^A^G^A=c|' -e 'D|EGE|G2D|EGE|G2D|EGE' -e '|G,ED|EG,G,|G,ED|' -e '|=d=c=e=d=c=e|=d=c=e=d=c=e|=d' \
          -e '|=E=G=G=E=G=A=G=F|=E=G=G=E=G=A=G=F|' -e 'E2EDECCD|E2EDECCD|E' -e '|D=ED=ED=ED2|D=ED=ED=ED2|D=ED' -e '|G,A,A,A,A,2G,G,|G,A,A' \
          -e '|=d=g=g=d=g=g|=d=g=g=d=g=g|=d' -e '|=d=g=g=d=g=g|=d=g=g=d=g=g' -e 'EGGAGEDC|EGGAGACD|E' -e '|(3DDDD2(3DDDD2|(3DDDD2(3DDDD2|' \
          -e '|G,B,CDDB,|G,B,CDDB,|G,B,CDDB,|' -e '|G,G,A,G,A,G,F,G,|G,G,A,G,A,G,F,' -e '|^G^A^G^G2^G|^G^A^G^G2^G|' -e '|^g^a^a^a^g=f|^g^a^a^a^g=f|' \
          -e '|=d=g=g=d=g=g|=d=g=g=d=g=g|' -e '|=e=d=e=d=e=d=e=d|=e=d=e=d=e=d=e=d|' -e '=c|=d=c=A=c=A=c|=d=c=A=c=d=d|' -e 'g|=d=g=g=b=g=g|=d=g=g=b=g=g|=d' -e "|=d=c'=a=g=d=e|=d=c'=a=g=d=e|=d" -e '=G|=A=c=A=A2=G|=A=c=A=A2=G|=A' -e '|=d=g=g=d=g=g|=d=g=g=d=g=g|' \
          -e '|=g=f=e=d=c=d=e=f|=g=f=e=d=c=d=e=f|' -e '|=A=G=A=C2=G|=A=G=A=C2=G|=A=G=A=C2=G|' -e '|^g^f^g^f^d^d|^g^f^g^f^d^d|' \
          -e 'G,|G,G,G,|G,G,G,|' -e 'G|=G=G=G=G=G=G=G=G|' -e '=G2|=G2=G2=G2=G2=G2|=G2' -e '|=d=g=g=d=g=g|=a=g=f=e=g=g|=d=g=g=d=g=g|=a=g=g=g=f=e|' \
          -e 'G,G,G,G,G,G,G,G,G,G,' -e '|=d=g=g=d=g=g|=a=g=f=e=g=g|=d=g=g=d=g=g|' -e '|=d=g=g=d=g=g|=a=g=g=g=f=e|=d=g=g=d' -e '|GGGGGGGGGGGGGGGGGGGG|' \
          -e '2G2G2G2G2G2G2G2G2' -e '|=G=G=G=G=G=G=G|=G=G=G=G=G=G=G|' -e 'EDEDEDEDEDEDEDEDEDEDEDED' -e '|=a=g=f=e=g=g|=d=g=g=d=g=g|=a=g=' \
          -e 'CA,CA,CA,CA,CA,CA,CA,CA,CA,' -e ',C,C,C,C,C,C,C,C,C,C,C,C,C,' -e 'BcB|BcBcBcB|BcB' -e '\n|\n|\n|\n|\n|' -e '2F2A2G2A2A2G2A2A2' \
          -e '2D2D2D2D2D2D2D2D2' -e '2c/2c/2c/2c/2c/2c/' -e '|GB\n|GB\n|GB\n|GB\n|GB\n|GB\n|' -e 'c/c/c/c/c/c/c/c/c/c/c' -e '/g/g/g/g/g/g/g/g/g' \
          -e 'A2G2A2G2A2G2A2G2A2A2G2A2G2A' -e 'EBEBEBEBEBEBEBEBEBEBEB' -e '|G2G2G2G2|G2A2G2A2|G2A2G2A2|' -e 'EDEDEDEDEDED' -e 'B,B,B,B,B,B,B' \
          -e 'B/B/B/B/B/B/B/B' -e 'G/G/G/G/G/G/G/G/' -e 'AcAcAcAcAcAcAcA' -e '^A|^A^A^A^A^A^A^A^A|^' -e '^D|^D^D^D^D^D^D^D|^' \
          -e '2g/2g/2g/2g/2g/2g/2g/2g/' -e '2d/2d/2d/2d/2d/2d/2d/2d/' -e '/2B/2B/2B/2B/2B/2B/2B' -e 'c/2c/2c/2c/2c/2c/2c/' \
          -e 'aaeaaeaaeaaeaaeaaea' -e '////////////' -e 'e/e/e/e/e/e/e/e/e/e' -e '222222222' -e 'cccccccccccccccccc' -e 'DADDADDADDA' \
          -e '4g/4a/4g/4a/4g/4a/4g' -e '2A2A2A2A2G2A2A2A2G2A2A2A2' -e 'fBfBfBfBfBfBfBfBfBfB' -e '2A2G2A2G2A2A2' -e '(3B)(3B)(3B)(3B)(3B)' \
          -e '\n\n\n' -e '-ghhathan-ghhathan-ghhathan' -e '|D2D2D2|D2D2D2|D2D2D2|' -e '|f/a/g/f/e/d/|f/a/g/f/e/d/|f/a/g/f/e/d/|f/a/g' \
          -e '|G2A2A2G2A2|G2A2G2A2G2' -e 'G2E2C2D2|G2E2C2D2|' -e '|c2ecgcec|c2ecgcec|' -e 'c2|c2|c2|c2|' -e 'g2d2e2d2e2d2|g2d2e2d2e2d2' \
          -e 'C2E2E2E2|D2F2D2D2|' -e 'E2F2E2E2E2F2|F2E2E2E2E2F2|' -e 'd2d2|d2d2|d2d2' -e '|f2f2|f2f2|f2f2|' -e '|c2A(A2B/d/)|c2A(A2B/d/)|' \
          -e 'd/B/d/B/d/B/d/B/|d/B/d/' -e '|F2E2E2E2|F2E2F2E2|' -e 'A2A2A2A2|A2A2A2A2|' -e '^g^g^f^g^g^g^g' -e 'E2E2E2E2E2E2|E2E2E2E2E2E2' \
          -e 'E8E8F8E8F8E8E8E8E8E8E8' -e 'fdfagfeg|fdfagfeg|' -e '|F2F2F2|F2F2F2|F2F2F2|' -e '|F2A2A2|G2B2B2|G2G2G2|' -e 'G2|G2|G2|G2|' \
          -e '|C2D|C2D|C2D|' -e 'c2c|c2c|c2c|' -e '|g2|g2|g2|g2|g2|' -e 'f2dg2d|f2dg2d|f2dg2d|' -e '|G2F2A2A2|G2F2A2A2|' \
          -e 'e2a2f2|e2f2e2e2e2a2f2|e2f' -e 'E2E2E2E2|E2E2E2E2|' -e '|EFD|EFD|EFD|' -e 'G2E2C2E2|G2E2C2E2|' -e '2C2C2C2C2' \
          -e '2F,2F,2F,2F,2F' -e 'G4F2E2F2F2|G4F2E2F2F2|' -e 'g2ef2d|g2ef2d|g2ef2d|' -e '|d2e2f2f2|f2e2e4|d2e2f2f2|f2e2e4|' \
          -e '|B2c2d2|B2c2d2|B2c2d2|' -e 'F2F2E2|F2F2E2|F2' -e 'F2FG/G/F2F2|F2FG/G/F2z2|F2FG' -e '|G3BG2G2G2G2|G3BG2G2G2G2|' \
          -e '|z4|z4|z4|z4|' -e 'G2A2F2F2|G2A2F2F2|' -e 'D4|D4|D4|D4|D4|' -e 'C2E2C2E2|C2E2C2E2' -e 'g2d2B2d2|g2f2d2B2' \
          -e '|G2EF|G2EF|G2EF|' -e 'F2F2F2|F2F2F2|F2F2' -e 'FE|DEDEFE|DEDEFE|' -e 'F2F2F2F2F2F2' -e '/B/d/B/d/B/d/B/' \
          -e 'E2F2G2|E2F2G2|E2' -e '2|c2d2e2f2|c2d2e2f2' -e '|C2C|C2C|C2C|C2F' -e '/4C/4C/4C/C/' -e '/4B,/4B,/4B,/4B,/4B' \
          -e 'd2f2d2f2|d2f2d2f2' -e 'f2df2g|f2df2g|f2' -e 'A2A2A2|A2A2A2' -e 'B2c2B2|B2c2B2|B2' -e 'C4E2C2|C4E2C2|C4E' \
          -e 'G2G2G2G2G' -e 'F2F2F2F2F2' -e 'E2E2E2E2A2' -e 'd2d2d2d2d2d2' -e 'B2G2d2|B2G2d2' -e 'G2G2G2G2|G2G2G2' \
          -e 'C2C2C2C2C2C' -e '|E2F2E2|E2F2E2|' -e 'G2G2|G2G2' -e 'g2gg2g|g2gg' -e '"A"e2f"Bb"d2c|"A"e2f"Bb"d2c|' \
          -e '2d2|"Bb"d2d2"Bb"d2d2|"Bb' -e 'B2B2B2B2|B2B2B2B2' -e 'f2d2d2e2f2g2|f2d2d2e2f2g2' -e '|"A"c2c2c2c2|"A"c2c2c2c2|' \
          -e '2f2e2|f2g2e2|f2g2e2|' -e '|B2G2B2B2G2B2|B2G2B2B2G2B2|' -e 'A2c2A2A2B2c2|A2c2A2A2B2c2' -e '|B2d2B2B2d2B2|B2d2B2B2d2B2' \
          -e '|c2e2c2c2e2c2|c2e2c2c2e2c2|' -e 'd2B2d2d2B2d2|d2B2d2d2B2d2|' -e '2|d2c2B2A2B2d2|d2c2B2A2B2d2|' \
          -e 'g4|g2f2g2e2|d2B2g4|g' -e 'f/g/f/g/f/g/f/g/' -e 'g2b2g2d2|e2g2d2B2|g2b' -e '|d/c/B/c/d/c/|d/c/B/c/d/c/' \
          -e 'f2g2|f2g2|f2g2' -e 'f2f2f2f2|f2' -e 'e2ee2e|e2ee2e|' -e 'B2A3d|B2A3d|B2' -e 'g2|g2|g2|g2|' -e 'd2g2d2g2b2d2g2b' \
          -e 'd2g2f2|g2f2e2|d2g2f2' -e 'g2f2g2g2|g2f2g2g2' -e 'g2d|g2d|' -e 'f2d|f2d|' -e '"g2"g2"g2"g2"g2"g2' -e 'c2c2c2|c2c2c2|' \
          -e 'd2d2"d2"c2|d2d2"d2"c2' -e 'g2d2c2B2|g2d2c2B2' -e 'g2bg2b|g2bg2b|g2'

}

echo "[" >> $JSON-encoded.json; echo "[" >> $JSON-text.json;

i=0
POEMS1=()
for file in $(ls /tmp/music-samples-1/sample-*.encoded | sed -e 's/\.encoded//' | tail -500)
do
    if [[ -f $file ]]; then
        POEMS1[$i]=$file
        i=$(($i+1))
    fi
done
i=0
POEMS2=()
for file in $(ls /tmp/music-samples-2/sample-*.encoded | sed -e 's/\.encoded//' | tail -500)
do
    if [[ -f $file ]]; then
        POEMS2[$i]=$file
        i=$(($i+1))
    fi
done
echo "${POEMS1[*]}" "${POEMS2[*]}"

# should we simply auto-rate all pieces where possible (skipping pairs where both are valid) and not ask for any manual ratings?
SKIP_ALL=""

for I in `seq 0 $N`; do
    echo "MUSIC: $I"

    POEM1=${POEMS1[$I]}
    FIRST=$(cat $POEM1)
    FIRST_ENCODED=$(cat $POEM1.encoded)

    POEM2=${POEMS2[$I]}
    SECOND=$(cat $POEM2)
    SECOND_ENCODED=$(cat $POEM2.encoded)

    # if first but not second is broken, second is the winner; if second but not first is broken, first wins;
    # and if both are broken, then we insert a pair where both win to tell the model that they are equally bad.
    # The check is a >100kb WAV file; if the ABC file is syntactically broken or too short to bother rating or
    # something goes wrong with abc2midi/timidity, then there will be no or small WAV files, so this checks most
    # errors. The other major error case is long repetitive degenerate ABC pieces generated by the model, so we
    # have a 'filterMusic' blacklist for snippets which show up in degenerate pieces.
    if [ ! $(wc -c < $POEM1.wav) -ge 100000 ] || [[ -n $(echo "$FIRST" | filterMusic) ]]; then
        generateJsonBroken 1 "$FIRST_ENCODED" "$SECOND_ENCODED" "$FIRST" "$SECOND"
    elif [ ! $(wc -c < $POEM2.wav) -ge 100000 ] || [[ -n $(echo "$SECOND" | filterMusic) ]]; then
        generateJsonBroken 0 "$FIRST_ENCODED" "$SECOND_ENCODED" "$FIRST" "$SECOND"
    elif [ ! $(wc -c < $POEM1.wav) -ge 100000 ] || [[ -n $(echo "$FIRST" | filterMusic) ]] && \
             ([ ! $(wc -c < $POEM2.wav) -ge 100000 ] || [[ -n $(echo "$SECOND" | filterMusic) ]]); then
        generateJson 1 "$FIRST_ENCODED" "$SECOND_ENCODED" "$FIRST" "$SECOND"
        generateJson 0 "$FIRST_ENCODED" "$SECOND_ENCODED" "$FIRST" "$SECOND"
    else
        if [ -z "$SKIP_ALL" ]; then
            echo -e "\n\e[1m--------------------------------------------\e[0m"
            FLIP="$((RANDOM % 2 < 1))"
            if [ $FLIP == 0 ]; then
            echo "$FIRST"
            timeout 20s mpv --af=scaletempo=scale=1.1:speed=pitch $POEM1.wav
            sleep 1s

            echo "============================================="
            echo "$SECOND"
            timeout 20s mpv --af=scaletempo=scale=1.1:speed=pitch $POEM2.wav
            echo "" # print a newline to make output easier to read and divide from the foregoing

            echo -e "[$I] 1: \e[1mFirst\e[0m wins | 2: \e[1mSecond\e[0m wins | 3: Equal | r: stop & auto-Rate Rest \
                    | x: e\e[1mX\e[0mit immediately"
            read -N 1 RATING
            case "$RATING" in

                $'\n')
                # skip
                ;;

                1)
                    generateJson 0 "$FIRST_ENCODED" "$SECOND_ENCODED" "$FIRST" "$SECOND"
                    ;;
                2)
                    generateJson 1 "$FIRST_ENCODED" "$SECOND_ENCODED" "$FIRST" "$SECOND"
                    ;;
                3)
                    generateJson 0 "$FIRST_ENCODED" "$SECOND_ENCODED" "$FIRST" "$SECOND"
                    generateJson 1 "$FIRST_ENCODED" "$SECOND_ENCODED" "$FIRST" "$SECOND"
                    ;;
                r)
                    SKIP_ALL="true"
                    ;;
                x)
                    break
                    ;;
            esac
            else
                echo "$FIRST"
                timeout 20s mpv --af=scaletempo=scale=1.1:speed=pitch $POEM1.wav
                sleep 1s

                echo "============================================="
                echo "$SECOND"
                timeout 20s mpv --af=scaletempo=scale=1.1:speed=pitch $POEM2.wav
                echo "" # print a newline to make output easier to read and divide from the foregoing

                echo -e "[$I] 1: \e[1mFirst\e[0m wins | 2: \e[1mSecond\e[0m wins | 3: Equal | r: stop & auto-Rate Rest \
                     | x: e\e[1mX\e[0mit immediately"
                read -N 1 RATING
                case "$RATING" in

                    $'\n')
                    # skip
                    ;;

                    1)
                        generateJson 1 "$FIRST_ENCODED" "$SECOND_ENCODED" "$FIRST" "$SECOND"
                        ;;
                    2)

                        generateJson 0 "$FIRST_ENCODED" "$SECOND_ENCODED" "$FIRST" "$SECOND"
                        ;;
                    3)
                        generateJson 0 "$FIRST_ENCODED" "$SECOND_ENCODED" "$FIRST" "$SECOND"
                        generateJson 1 "$FIRST_ENCODED" "$SECOND_ENCODED" "$FIRST" "$SECOND"
                        ;;
                    r)
                        SKIP_ALL="true"
                        ;;
                    x)
                        break
                        ;;
                esac
            fi
        fi
    fi
    I=$((I+1))
done
echo "]" >> $JSON-text.json; echo "]" >> $JSON-encoded.json
